-- Implements a circuit that can display 3-letter words on the six 
-- 7-segment displays. The character selected for each display is chosen by
-- a multiplexer, and these multiplexers are connected to the characters
-- in a way that allows a word to be rotated across the displays from
-- right-to-left as the multiplexer select lines are changed through the
-- sequence 000, 001, ..., 101, 000, etc. Using the four characters d,
-- E, 1, and "blank" the displays can show any 3-letter word using 
-- these letters, such as "dE1 ", as follows:
--
-- SW  987      Displayed characters
--     000         dE1
--     001        dE1
--     010       dE1
--     011      dE1
--     100      E1   d
--     101      1   dE
--
-- inputs:  SW9-7 provide the multiplexer select lines
--          SW5-0 provide three different codes used to select characters
-- outputs: LEDR shows the states of the switches
--          HEX5 - HEX0 displays the characters
LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY part6 IS 
   PORT ( SW   : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
          LEDR : OUT STD_LOGIC_VECTOR(9 DOWNTO 0);
          HEX5, HEX4, HEX3, HEX2, HEX1, HEX0 : OUT STD_LOGIC_VECTOR(0 TO 6));
END part6;

ARCHITECTURE Structure OF part6 IS
   COMPONENT mux_2bit_6to1
      PORT ( S : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
             U, V, W, X, Y, Z : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
             M : OUT STD_LOGIC_VECTOR(1 DOWNTO 0));
   END COMPONENT;
   COMPONENT char_7seg
      PORT ( C       : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
             Display : OUT STD_LOGIC_VECTOR(0 TO 6));
   END COMPONENT;
   SIGNAL Ch_Sel :   STD_LOGIC_VECTOR(2 DOWNTO 0);
   SIGNAL Blank, Ch1, Ch2, Ch3 :   STD_LOGIC_VECTOR(1 DOWNTO 0);
   SIGNAL H5_Ch, H4_Ch, H3_Ch, H2_Ch, H1_Ch, H0_Ch : STD_LOGIC_VECTOR(1 DOWNTO 0);
BEGIN
   LEDR <= SW;
   Ch_Sel <= SW(9 DOWNTO 7);
   Blank <= "11";   -- used to blank a 7-seg display (see module char_7seg)
   Ch1 <= "00"; -- 'd'
   Ch2 <= "01"; -- 'E'
   Ch3 <= "10"; -- '1'

   -- instantiate entity mux_2bit_4to1 (S, U, V, W, X, Y, Z, M) to
   -- create the multiplexer for each hex display
   M5: mux_2bit_6to1 PORT MAP (Ch_Sel, Blank, Blank, Blank, Ch1, Ch2, Ch3, H5_CH);
   M4: mux_2bit_6to1 PORT MAP (Ch_Sel, Blank, Blank, Ch1, Ch2, Ch3, Blank, H4_CH);
   M3: mux_2bit_6to1 PORT MAP (Ch_Sel, Blank, Ch1, Ch2, Ch3, Blank, Blank, H3_CH);
   M2: mux_2bit_6to1 PORT MAP (Ch_Sel, Ch1, Ch2, Ch3, Blank, Blank, Blank, H2_CH);
   M1: mux_2bit_6to1 PORT MAP (Ch_Sel, Ch2, Ch3, Blank, Blank, Blank, Ch1, H1_CH);
   M0: mux_2bit_6to1 PORT MAP (Ch_Sel, Ch3, Blank, Blank, Blank, Ch1, Ch2, H0_CH);

   -- instantiate module char_7seg (C, Display) to drive the hex displays
   H5: char_7seg PORT MAP (H5_Ch, HEX5);
   H4: char_7seg PORT MAP (H4_Ch, HEX4);
   H3: char_7seg PORT MAP (H3_Ch, HEX3);
   H2: char_7seg PORT MAP (H2_Ch, HEX2);
   H1: char_7seg PORT MAP (H1_Ch, HEX1);
   H0: char_7seg PORT MAP (H0_Ch, HEX0);
END Structure;

LIBRARY ieee;
USE ieee.std_logic_1164.all;

-- implements a 2-bit wide 6-to-1 multiplexer
ENTITY mux_2bit_6to1 IS
   PORT ( S : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
          U, V, W, X, Y, Z : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
          M : OUT STD_LOGIC_VECTOR(1 DOWNTO 0));
END mux_2bit_6to1;

ARCHITECTURE Behavior OF mux_2bit_6to1 IS
   SIGNAL m_0, m_1   : STD_LOGIC_VECTOR(1 TO 4); -- intermediate multiplexers
BEGIN
   -- 6-to-1 multiplexer for bit 0
   m_0(1) <= (NOT(S(0)) AND U(0)) OR (S(0) AND V(0));
   m_0(2) <= (NOT(S(0)) AND W(0)) OR (S(0) AND X(0));
   m_0(3) <= (NOT(S(0)) AND Y(0)) OR (S(0) AND Z(0));
   m_0(4) <= (NOT(S(1)) AND m_0(1)) OR (S(1) AND m_0(2));
   
   M(0) <= (NOT(S(2)) AND m_0(4)) OR (S(2) AND m_0(3));

   -- 6-to-1 multiplexer for bit 1
   m_1(1) <= (NOT(S(0)) AND U(1)) OR (S(0) AND V(1));
   m_1(2) <= (NOT(S(0)) AND W(1)) OR (S(0) AND X(1));
   m_1(3) <= (NOT(S(0)) AND Y(1)) OR (S(0) AND Z(1));
   m_1(4) <= (NOT(S(1)) AND m_1(1)) OR (S(1) AND m_1(2));
   
   M(1) <= (NOT(S(2)) AND m_1(4)) OR (S(2) AND m_1(3));

END Behavior;

LIBRARY ieee;
USE ieee.std_logic_1164.all;

-- Converts 2-bit input code on C1-0 into 7-bit code that produces
-- a character on a 7-segment display. The conversion is defined by:
--     C  1 0   Char
--    ----------------
--        0 0 'd'
--        0 1   'E'
--        1 0 '1'
--        1 1   ' ' blank
--
--
ENTITY char_7seg IS
   PORT ( C       : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);
          Display : OUT STD_LOGIC_VECTOR(0 TO 6));
END char_7seg;
   --
   --       0  
   --      ---  
   --     |   |
   --    5|   |1
   --     | 6 |
   --      ---  
   --     |   |
   --    4|   |2
   --     |   |
   --      ---  
   --       3  
   --
ARCHITECTURE Behavior OF char_7seg IS
BEGIN
   -- the following equations describe display functions in (inverted) cannonical SOP form
   Display(0) <= NOT( NOT(C(1)) AND C(0)); 
   Display(1) <= C(0); 
   Display(2) <= C(0); 
   Display(3) <= C(1); 
   Display(4) <= C(1);
   Display(5) <= NOT( NOT(C(1)) AND C(0)); 
   Display(6) <= C(1);   
END Behavior;
